/*
    Copyright (C) 2005-2007  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <glib-object.h>
#include <glib/gi18n.h>

#include "nntpgrab_plugin_schedular.h"
#include "schedular_plugin.h"

static PluginSchedularImportedFuncs imported_funcs;

void
schedular_plugin_set_imported_funcs(PluginSchedularImportedFuncs funcs)
{
    imported_funcs = funcs;
}

void
imported_funcs_task_added (const NNTPCollection *collection, const NNTPFile *file)
{
    imported_funcs.task_added(collection, file);
}

void
imported_funcs_task_removed (const char *collection_name, const char *subject, guint64 total_file_size, guint64 total_file_size_remaining)
{
    imported_funcs.task_removed(collection_name, subject, total_file_size, total_file_size_remaining);
}

void
imported_funcs_task_state_changed (const char *collection_name, const char *subject, TaskState old_state, TaskState new_state)
{
    imported_funcs.task_state_changed(collection_name, subject, old_state, new_state);
}

void
imported_funcs_collection_added (const char *collection_name)
{
    imported_funcs.collection_added(collection_name);
}

void
imported_funcs_collection_removed (const char *collection_name)
{
    imported_funcs.collection_removed(collection_name);
}

void
imported_funcs_notify_schedular_state_changed (SchedularState new_state, const char *reason)
{
    imported_funcs.notify_schedular_state_changed(new_state, reason);
}

GList *
imported_funcs_config_get_avail_servers (void)
{
    return imported_funcs.config_get_avail_servers();
}

ConfigServer *
imported_funcs_config_get_server_info (const char *servername)
{
    return imported_funcs.config_get_server_info(servername);
}

ConfigOpts
imported_funcs_config_get_opts (void)
{
    return imported_funcs.config_get_opts();
}

gpointer
imported_funcs_nntp_connection_initialize (const char *servername, NNTPGrabErrCode *err)
{
    return imported_funcs.nntp_connection_initialize(servername, err);
}

void
imported_funcs_nntp_connection_destroy (gpointer obj)
{
    imported_funcs.nntp_connection_destroy(obj);
}

NNTPGrabErrCode
imported_funcs_nntp_connection_get_part (gpointer obj, const char *collection_name, const NNTPFile *file, const NNTPPart *part, char **reason)
{
    return imported_funcs.nntp_connection_get_part(obj, collection_name, file, part, reason);
}

gboolean
imported_funcs_decode_file (const char *collection_name, const NNTPFile *file, int *saved_errno)
{
    return imported_funcs.decode_file(collection_name, file, saved_errno);
}

void
imported_funcs_emit_fatal_error (const char *errmsg)
{
    imported_funcs.emit_fatal_error(errmsg);
}

void
imported_funcs_emit_part_download_complete (const char *servername, gpointer nntp_connection, const char *collection_name, const char *subject, int partnum, int size)
{
    imported_funcs.emit_part_download_complete(servername, nntp_connection, collection_name, subject, partnum, size);
}

void
imported_funcs_emit_part_download_failed (const char *servername, gpointer nntp_connection, const char *collection_name, const char *subject, int partnum, int size, gboolean all_servers_tried)
{
    imported_funcs.emit_part_download_failed(servername, nntp_connection, collection_name, subject, partnum, size, all_servers_tried);
}

void
imported_funcs_emit_task_moved (const char *orig_collection_name, const char *subject, const char *new_collection_name, int old_position, int new_position)
{
    imported_funcs.emit_task_moved (orig_collection_name, subject, new_collection_name, old_position, new_position);
}

void
imported_funcs_emit_collection_moved (const char *collection_name, int old_position, int new_position)
{
    imported_funcs.emit_collection_moved (collection_name, old_position, new_position);
}

void
imported_funcs_emit_file_download_state_update (const char *collection_name, const char *subject, int num_parts_total, int num_parts_done, int num_parts_failed, guint64 file_size, guint64 file_size_remaining, guint64 total_file_size, guint64 total_file_size_remaining)
{
    imported_funcs.emit_file_download_state_update (collection_name, subject, num_parts_total, num_parts_done, num_parts_failed, file_size, file_size_remaining, total_file_size, total_file_size_remaining);
}
